from . import db, login_manager
from datetime import datetime
from flask_login import UserMixin
from werkzeug.security import generate_password_hash, check_password_hash
from itsdangerous import TimedJSONWebSignatureSerializer as Serializer
from flask import current_app, url_for


@login_manager.user_loader
def load_user(user_id):
    return User.query.get(int(user_id))


roles = db.Table(
    'role_users',
    db.Column('user_id', db.Integer, db.ForeignKey('user.id')),
    db.Column('role_id', db.Integer, db.ForeignKey('role.id'))
)


class User(UserMixin, db.Model):
    id = db.Column(db.Integer(), primary_key=True)
    username = db.Column(db.String(64), nullable=False, index=True, unique=True)
    email = db.Column(db.String(64), nullable=False, unique=True)
    password_hash = db.Column(db.String(128), nullable=False)
    confirmed = db.Column(db.Boolean, default=False)
    roles = db.relationship(
        'Role',
        secondary=roles,
        backref=db.backref('users', lazy='dynamic')
    )
    # transactions = db.relationship('Transaction', backref='user', lazy='subquery')

    def __init__(self, email, username, password_hash):
        self.email = email
        self.username = username
        self.password(password_hash)

    def __repr__(self):
        return "<User '{}'>".format(self.username)

    def password(self, password):
        self.password_hash = generate_password_hash(password)

    def verify_password(self, password):
        return check_password_hash(self.password_hash, password)

    def generate_confirmation_token(self, expiration=3600):
        s = Serializer(current_app.config['SECRET_KEY'], expiration)
        return s.dumps({'confirm': self.id}).decode('utf-8')

    def confirm(self, token):
        s = Serializer(current_app.config['SECRET_KEY'])
        try:
            data = s.loads(token.encode('utf-8'))
        except:
            return False
        if data.get('confirm') != self.id:
            return False
        self.confirmed = True
        db.session.add(self)
        return True

    def has_role(self, name):
        for role in self.roles:
            if role.name == name:
                return True
        return False

    def send_email_confirmation(self, email, token):
        import requests
        api_address = "https://api.mailgun.net/v3/mg.ratchfordconsulting.com/messages"
        api_key = "a8cbcda28c7cee6eabe23c1ab9eae61a-2b778fc3-d2dcbd67"
        _from = "MinDrive@ratchfordconsulting.com"
        _to = email
        _subject = "Please confirm your {} account".format(_from)
        _link = url_for("auth.confirm_email", user_id=self.id, token=token)
        _text = "Please follow the link below to confirm your {} account and log in. \n\n{}".format(_from, _link)
        requests.post(api_address,
                      auth=("api", api_key),
                      data={"from": _from,
                            "to": _to,
                            "subject": _subject,
                            "text": _text
                            }
                      )


class Role(db.Model):
    id = db.Column(db.Integer(), primary_key=True)
    name = db.Column(db.String(80), unique=True)
    description = db.Column(db.String(255))

    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return '<Role {}>'.format(self.name)



# todo Role Class / UserRole Class


class File(db.Model):
    id = db.Column(db.Integer(), primary_key=True)
    filename = db.Column(db.String(255), index=True, nullable=False)
    parent_dir = db.Column(db.String(255), index=True)
    full_path = db.Column(db.String(255), unique=True)
    file_ext = db.Column(db.String(255))
    file_size = db.Column(db.Integer(), nullable=False)
    is_dir = db.Column(db.Boolean())
    last_index_time = db.Column(db.DateTime())
    create_time = db.Column(db.DateTime())
    last_access_time = db.Column(db.DateTime())
    last_write_time = db.Column(db.DateTime())
    last_attr_change_time = db.Column(db.DateTime())
    alloc_size = db.Column(db.Integer())
    file_attributes = db.Column(db.Integer())
    is_normal = db.Column(db.Boolean())
    is_read_only = db.Column(db.Boolean())

    transaction = db.relationship('Transaction', backref='file', lazy='subquery')

    def __init__(self, filename, parent_dir, full_path, file_ext,
                 file_size, is_dir, last_index_time, create_time,
                 last_access_time, last_write_time, last_attr_change_time,
                 alloc_size, file_attributes, is_normal, is_read_only):

        self.filename = filename
        self.parent_dir = parent_dir
        self.full_path = full_path
        self.file_ext = file_ext
        self.file_size = file_size
        self.is_dir = is_dir
        self.last_index_time = last_index_time
        self.create_time = create_time
        self.last_access_time = last_access_time
        self.last_write_time = last_write_time
        self.last_attr_change_time = last_attr_change_time
        self.alloc_size = alloc_size
        self.file_attributes = file_attributes
        self.is_normal = is_normal
        self.is_read_only = is_read_only

    def __repr__(self):
        return "<File '{}'".format(self.filename)


class Transaction(db.Model):
    id = db.Column(db.Integer(), primary_key=True)
    action = db.Column(db.String(255))
    date = db.Column(db.DateTime(), default=datetime.now)

    user_id = db.Column(db.Integer(), db.ForeignKey('user.id'))
    file_id = db.Column(db.Integer(), db.ForeignKey('file.id'))

    def __repr__(self):
        return "<Comment '{}'>".format(self.text[:15])
